/* DM_Comm_Functions is the C file that contains all of the functions neccessary to operate the 
DM_Comm.c file. This includes initialization functions as well as implementation functions.
Written by Barrett Taylor on July 3, 2013.
*/

#include "DM_Headers.h"

uint32_t g_pixel_count;

/**********************************************************************************************
void RunComms(): Is basically like main() in that this calls all the other functions.
Needed to make a separate function to be able ot call it from timeout routines.
**********************************************************************************************/
void RunComms(void)
{
	uint16_t 			OCE_retreived_data[2];									//Variable to hold 16 bit data piece from OCELOT.
	uint16_t* 		OCE_retreived_data_ptr;									//Pointer to the OCELOT retreived data.
	uint16_t			DM_retreived_data[2];										//Variable to hold 16 bit data from the DM.
	uint16_t*			DM_retreived_data_ptr;									//Pointer to data received from the DM.
	bool					go;
	
	OCE_retreived_data_ptr 	= &OCE_retreived_data[0];			//Give the address of the data to the pointer.
	DM_retreived_data_ptr 	= &DM_retreived_data[0];			//Give the address of the data to the pointer.
	g_pixel_count 					= 0;													//Set the pixel count to zero at the start of every communication.
	go 											= true;
	
	//while (1)																							//Loop forever, continually checking for inputs.
	while (go)
	{
		//ReceiveCommand(OCE_retreived_data_ptr);							//Get a command from the receive FIFO from OCELOT.
		OCE_retreived_data[0] = 0x0003;
		if (OCE_retreived_data[0] == 0x0003)								//Check if the start command was just sent.
		{
			//SendCommand(OCE_retreived_data_ptr);							//Send the retreived command to OSIRIS.
			GetImage(DM_retreived_data_ptr);									//Get the image back from the DM and send to OCELOT.
			go = false;
		}
		else
		{
			EchoCommand(OCE_retreived_data_ptr);							//Send the received command back to OCELOT to confirm.
			SendCommand(OCE_retreived_data_ptr);							//Send the retreived command to OSIRIS.
		}
	}
}

/**********************************************************************************************
void ReceiveCommand(): This functions gets the data from the receive FIFO sent from OCELOT.
**********************************************************************************************/
void ReceiveCommand(uint16_t* data_ptr)
{
	SSIDataGet(SSI0_BASE, (uint32_t *)(data_ptr));									//Get a command from the receive FIFO if there is one. If there isn't it will wait for one.
}

/**********************************************************************************************
void EchoCommand(): This functions echos the received command from OCELOT back to it.
**********************************************************************************************/
void EchoCommand(uint16_t* echo_command)
{
	uint32_t				garbage[1];																			//Variable to hold the extra data sent over SPI when OCE receives the echo.
	uint32_t*				garbage_ptr;																		//Pointer to the garbage buffer.
	
	garbage_ptr = &garbage[0];																			//Assign the pointer the buffer address.
	SSIDataPut(SSI0_BASE, echo_command[0]);													//Put the echo'd command into the FIFO.
	GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_4, 0x10);								//Send a flag high to tell OCELOT to expect data.
	while (SSIBusy(SSI0_BASE));																			//Wait for the echo to be sent back.
	SSIDataGet(SSI0_BASE, garbage_ptr);															//After sending the echo, get the garbage command out of the receive FIFO.
}

/**********************************************************************************************
void SendCommand(): This functions sends the required commands to the DM to set it up to get the
appropriate image. It sets the exposure time constant, the exposure exponent and the vector.
**********************************************************************************************/
void SendCommand(uint16_t* data)
{
	SSIDataPut(SSI1_BASE, data[0]);																	//Put the received command into the SSI FIFO to be sent out.
	TimerEnable(TIMER1_BASE, TIMER_B);															//Enable (start) the synchronization timer.
	while(SSIBusy(SSI1_BASE));																			//Wait for the SSI to send out the entire command before doing anything else.
}

/**********************************************************************************************
void GetImage(): This functions receives data from OSIRIS and sends it to OCELOT one word at
a time.
**********************************************************************************************/
void GetImage(uint16_t* image)
{
	TimerEnable(WTIMER0_BASE, TIMER_B);															//Start the timeout timer in case full image is not received.
	//uDMAChannelEnable(14);																					//Enable channel 14 DMA to start receiving data.
	while(g_pixel_count < (144*1369));																//Get data until the expected number of pixels have been received.	
	
	uDMAChannelDisable(14);
	TimerDisable(WTIMER0_BASE, TIMER_B);
	SSIDataPut(SSI0_BASE, 0x8000);
	GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_4, 0x10);						//Send a flag high to tell the master to expect data.
	while (SSIBusy(SSI0_BASE));																	//Wait for the SSI to finish sending.
	GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_4, 0x00);						//Clear the flag sent to the master.
	SysCtlReset();
}

/**********************************************************************************************
void __irq TIMER1B_Handler(): This function is the interrupt handler that is called when the
synchronization timer triggers. The job of this interrupt is to control the DA line going to
the DM by dropping the line low in sync with the 2MHz clock.
**********************************************************************************************/
void __irq TIMER1B_Handler(void)
{
	TimerDisable(TIMER1_BASE, TIMER_B);													//Disable the synchronization timer.
	TimerIntClear(TIMER1_BASE, TIMER_CAPB_MATCH);								//Clear the interrupt flag for this timer.
	GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_3, 0x00);						//Drop the GPIO pin that is DA to the DM low.
	TimerEnable(TIMER2_BASE, TIMER_A);													//Enable the timer to count 16 edges before putting the line high again to end transmission.
}

/**********************************************************************************************
void __irq TIMER2A_Handler(): This function is the interrupt handler that is called when the
counter timer triggers. The job of this interrupt is to control the DA line going to the DM.
**********************************************************************************************/
void __irq TIMER2A_Handler(void)
{
	TimerDisable(TIMER2_BASE, TIMER_A);										//Disable the timer to allow it to be reset.
	TimerIntClear(TIMER2_BASE, TIMER_CAPA_MATCH);					//Clear the interrupt flag to allow it to trigger again.
	GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_3, 0x08);			//Set the DA line going to OSIRIS high again.
}

/**********************************************************************************************
void __irq TIMER1B_Handler(): This function is the interrupt handler that is called when the
counter timer triggers. The job of this interrupt is to control the DA line going to the DM.
**********************************************************************************************/
void __irq WTIMER0B_Handler(void)
{
	uDMAChannelDisable(14);
	TimerDisable(WTIMER0_BASE, TIMER_B);												//Disable the timer to allow it to be reset.
	TimerIntClear(WTIMER0_BASE, TIMER_CAPB_MATCH);							//Clear the interrupt flag to allow it to trigger again.

	SSIDataPut(SSI0_BASE, 0x8100);														//Send done transmission with timeout error.

	GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_4, 0x10);						//Send a flag high to tell the master to expect data.
	while (SSIBusy(SSI0_BASE));																	//Wait for the SSI to finish sending.
	GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_4, 0x00);						//Clear the flag sent to the master.
	SysCtlReset();
}

/**********************************************************************************************
void uDMA_Error_Interrupt(): This function is the error interrupt handler for the uDMA.
**********************************************************************************************/
void __irq UDMAERR_Handler(void)
{
	uint32_t status;
	
	status = uDMAErrorStatusGet();	
	if (status)
	{
		uDMAErrorStatusClear();
		uDMAChannelDisable(14);
		TimerDisable(TIMER1_BASE, TIMER_B);
		SSIDataPut(SSI0_BASE, 0x8200);
		GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_4, 0x10);
		SSIBusy(SSI0_BASE);
		GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_4, 0x00);
		SysCtlReset();
	}
}
/**********************************************************************************************
void __irq SSI3_Handler(): Is the interrupt handler for SSI3 whihc runs DMA Rx.
**********************************************************************************************/
void __irq SSI3_Handler(void)
{
	uint32_t status;
	uint32_t mode;
	uint16_t i;
	
	status = SSIIntStatus(SSI3_BASE, 1);
	SSIIntClear(SSI3_BASE, status);
	mode = uDMAChannelModeGet(14 | UDMA_PRI_SELECT);
	if (mode == UDMA_MODE_STOP)
	{
		for (i = 0; i < 8; i++)
		{
			SSIDataPut(SSI0_BASE, g_ROERxBufA[i]);
			g_pixel_count++;																						//Increment the pixel count.
		}
		GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_4, 0x10);							//Send a flag high to tell the master to expect data.
		while (SSIBusy(SSI0_BASE));																		//Wait for the SSI to finish sending.
		GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_4, 0x00);							//Clear the flag sent to the master.
		uDMAChannelTransferSet(14 | UDMA_PRI_SELECT,
			UDMA_MODE_PINGPONG, (void *) (SSI3_BASE + SSI_O_DR), g_ROERxBufA, 4);
	}
	
	mode = uDMAChannelModeGet(14 | UDMA_ALT_SELECT);
	
	if (mode == UDMA_MODE_STOP)
	{
		for (i = 0; i < 8; i++)
		{
			SSIDataPut(SSI0_BASE, g_ROERxBufB[i]);
			g_pixel_count++;																						//Increment the pixel count.
		}
		GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_4, 0x10);							//Send a flag high to tell the master to expect data.
		while (SSIBusy(SSI0_BASE));																		//Wait for the SSI to finish sending.
		GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_4, 0x00);							//Clear the flag sent to the master.
		uDMAChannelTransferSet(14 | UDMA_ALT_SELECT,
			UDMA_MODE_PINGPONG, (void *) (SSI3_BASE + SSI_O_DR), g_ROERxBufB, 4);
	}
	if(!uDMAChannelIsEnabled(14))
	{
		uDMAChannelEnable(14);
	}
}
